/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.plaf.basic.*;
import org.greenstone.gatherer.Gatherer;

/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version
 */
public class GComboBox
    extends JComboBox {

    private Color background = null;
    private Color foreground = null;
    private Color selection_background = null;
    private Color selection_foreground = null;

    public GComboBox() {
	super();
	init();
    }

    public GComboBox(ArrayList data) {
	super(data.toArray());
	init();
    }

    public GComboBox(ComboBoxModel model) {
	super(model);
	init();
    }

    public GComboBox(Gatherer gatherer) {
	super();
	init();
    }

    public GComboBox(Object data[]) {
	super(data);
	init();
    }

    public GComboBox(Vector data) {
	super(data);
	init();
    }

    public int add(Object object) {
	if(dataModel instanceof Model) {
	    return ((Model)dataModel).add(object);
	}
	else {
	    return -1;
	}
    }

    public Object get(int index) {
	return dataModel.getElementAt(index);
    }

    public void clear() {
	if(dataModel instanceof Model) {
	    ((Model)dataModel).clear();
	}
    }

    public int count() {
	return dataModel.getSize();
    }

    public void setBackgroundNonSelectionColor(Color background) {
	this.background = background;
	setEditor(new Editor());
	setRenderer(new Renderer());
    }

    public void setTextNonSelectionColor(Color foreground) {
	this.foreground = foreground;
	setEditor(new Editor());
	setRenderer(new Renderer());
    }

    public void setBackgroundSelectionColor(Color selection_background) {
	this.selection_background = selection_background;
	setEditor(new Editor());
	setRenderer(new Renderer());
    }

    public void setTextSelectionColor(Color selection_foreground) {
	this.selection_foreground = selection_foreground;
	setEditor(new Editor());
	setRenderer(new Renderer());
    }

    public void init() {
	Model model = new Model();
	ComboBoxModel old_model = (ComboBoxModel) getModel();
	setModel(model);
	// Restore any data given into our model
	for(int i = 0; i < old_model.getSize(); i++) {
	    model.add(old_model.getElementAt(i));
	}
	// Change component
	UI ui = new UI();
	setUI(ui);
	ui.setButtonBackground();
	// Initialization
	this.background = Gatherer.config.getColor("coloring.collection_tree_background", false);
	this.foreground = Gatherer.config.getColor("coloring.collection_tree_foreground", false);
	this.selection_background = Gatherer.config.getColor("coloring.collection_selection_background", false);
	this.selection_foreground = Gatherer.config.getColor("coloring.collection_selection_foreground", false);
	setEditor(new Editor());
	setRenderer(new Renderer());
    }

    private class Editor
	extends JTextField
	implements ComboBoxEditor {
	public Editor() {
	    setBackground(background);
	    setForeground(foreground);
	    setSelectionColor(selection_background);
	    setSelectedTextColor(selection_foreground);
	}

	public Component getEditorComponent() {
	    return this;
	}

	public Object getItem() {
	    return getText();
	}

	public void setItem(Object item) {
	    if(item != null) {
		setText(item.toString());
	    }
	    else {
		setText("");
	    }
	}
    }

    private class Model
	extends DefaultComboBoxModel {

	public int add(Object extension) {
	    int position = 0;
	    String extension_str = extension.toString().toLowerCase();
	    while(extension != null && position < getSize()) {
		String sibling = getElementAt(position).toString().toLowerCase();
		int order = extension_str.compareTo(sibling);
		// If we are now less than the sibling, insert here.
		if(order < 0) {
		    insertElementAt(extension, position);
		    extension = null;
		}
		// We are equal to the sibling, thus we already exist in list. Done.
		else if(order == 0) {
		    extension = null;
		}
		// Continue searching.
		else {
		    position++;
		}
	    }
	    if(extension != null) {
		position = getSize();
		addElement(extension);
	    }
	    return position;
	}

	public void clear() {
	    removeAllElements();
	}
    }

    private class Renderer
	extends JLabel
	implements ListCellRenderer {
	public Renderer() {
	}
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
	    if(value != null) {
		setText(value.toString());
	    }
	    else {
		setText("");
	    }
	    setBackground(isSelected ? selection_background : background);
	    setForeground(isSelected ? selection_foreground : foreground);
	    setOpaque(true);
	    return this;
	}
    }

    private class UI
	extends BasicComboBoxUI {
	public UI() {
	    super();
	}

	protected ComboPopup createPopup() {
	    BasicComboPopup popup = new BasicComboPopup( comboBox ) {
		    public void show() {
			if(comboBox.getMaximumRowCount() > 0) {
			    Dimension popupSize = new Dimension( comboBox.getWidth(), getPopupHeightForRowCount( comboBox.getMaximumRowCount()));
			    Rectangle popupBounds = new Rectangle( 0, 0, popupSize.width, popupSize.height);
			    scroller.setMaximumSize( popupBounds.getSize() );
			    scroller.setPreferredSize( popupBounds.getSize() );
			    scroller.setMinimumSize( popupBounds.getSize() );
			    list.invalidate();
			    int selectedIndex = comboBox.getSelectedIndex();
			    if ( selectedIndex == -1 ) {
				list.clearSelection();
			    }
			    else {
				list.setSelectedIndex( selectedIndex );
			    }
			    list.ensureIndexIsVisible( list.getSelectedIndex() );
			    setLightWeightPopupEnabled( comboBox.isLightWeightPopupEnabled() );
			    show(arrowButton, arrowButton.getSize().width - (popupSize.width + 4), arrowButton.getSize().height);
			}
		    }
		};
	    popup.getAccessibleContext().setAccessibleParent(comboBox);
	    return popup;
	}

	public void setButtonBackground() {
		arrowButton.setBackground(Gatherer.config.getColor("coloring.button_background", false));
	}
    }
}
